const config = require('config')
const needle = require("needle")
const db = require('machankura.db');
const lightninNodeUtil = require("../lightning-node-util")

const bitrefillAuthorization = config.get("bitrefill.authorization")

module.exports.lookupNumber = async (phoneNumber) => {
    if (process.env.NODE_ENV == "test") {
        // TODO: Look at what the input is
        return {
            operator: {
                slug: "slug"
            },
            country: {
                alpha2: "ZA"
            }
        }
    }
    const bitrefillCountryURL = `https://www.bitrefill.com/api/widget/lookup_number?number=${phoneNumber}`

    return needle(
        "GET",
        bitrefillCountryURL,
        {
            json: true
        }
    ).then(response => {
        if (response.statusCode == 200) {
            return response.body
        } else {
            console.error(`Failed to lookup phoneNumber ${phoneNumber}: `, response.statusMessage)
            // throw Error(response.statusMessage)
            return null
        }
    }).catch(error => {
        console.error("Failed to lookup number response: ", error)
        return null
    })
}

module.exports.lookupInvoice = async (invoiceId) => {
    const bitrefillInvoiceURL = `https://api-bitrefill.com/v2/invoices/${invoiceId}`

    return needle(
        "GET",
        bitrefillInvoiceURL,
        {
            json: true,
            headers: {
                "Authorization": bitrefillAuthorization
            }
        }
    ).then(response => {
        if (response.statusCode == 200) {
            return response.body
        } else {
            console.error("Failed to lookup invoice: ", response.statusMessage)
            return null
        }
    }).catch(error => {
        console.error("Failed to lookup invoice: ", error)
        return null
    })
}

const createBitrefillInvoice = (data) => {
    const bitrefillInvoiceURL = `https://api-bitrefill.com/v2/invoices`

    return needle(
        'post',
        bitrefillInvoiceURL, 
        data, 
        {
            json: true,
            headers: {
                "Authorization": bitrefillAuthorization
            }
        }
    )
}

const payBitrefillInvoice = (bitrefillInvoice) => {
    const bitrefillInvoicePayURL = `https://api-bitrefill.com/v2/invoices/${bitrefillInvoice.id}/pay`

    return needle(
        'post',
        bitrefillInvoicePayURL, 
        {}, 
        {
            json: true,
            headers: {
                "Authorization": bitrefillAuthorization
            }
        }
    )
}

module.exports.invoiceAirtime = async (user, sessionId, bitrefillProduct, phoneNumber, bitrefillInput) => {
    if (process.env.NODE_ENV == "test") {
        // TODO: Look at what the input is
        if (bitrefillInput.amount == 55 || bitrefillInput.amount == 550) {
            return await db.BitrefillInvoice.create({
                id: "guid",
                status: "not_yet_delivered",
                order: [],
                payment: {
                    method: "balance",
                    price: 1000
                },
                sessionId: sessionId,
                createdAt: new Date(),
                bitrefillProductId: bitrefillProduct.id,
                completedAt: null
            })
        }
        return null 
    }
    
    const data = {
        products: [
            {
                "product_id": bitrefillProduct.id,
                "quantity": 1,
                "phone_number": phoneNumber,
                "send_sms": true
            }
        ],
        "auto_pay": false,
        "payment_method": "balance"
    }

    if (bitrefillInput.amount) {
        data.products[0].value = bitrefillInput.amount
    } else if (bitrefillInput.packageId) {
        data.products[0].package_id = bitrefillInput.packageId
    }

    return createBitrefillInvoice(data)
    .then(async (response) => {
        if (response.statusCode == 200) {
            // TODO: Save bitrefillInvoice...
            if (user == null) {
                return null
            }
            return db.BitrefillInvoice.create({
                id: response.body.data.id,
                status: response.body.data.status,
                order: response.body.data.orders,
                sessionId: sessionId,
                payment: response.body.data.payment,
                createdAt: new Date(response.body.data.created_time),
                completedAt: response.body.data.completed_time ? new Date(response.body.data.completed_time) : null,
                bitrefillProductId: bitrefillProduct.id
            })
        } else {
            return null
        } 
    }).catch(error => {
        console.error("Failed to lookup number response: ", error)
        return null
    })
}

module.exports.buyAirtime = async (user, bitrefillInvoice, bitrefillInput) => {
    if (process.env.NODE_ENV == "test") {
        // TODO: Look at what the input is
        if (bitrefillInput.amount == 55 || bitrefillInput.amount == 550) {
            const transfer = await persistTransfer(
                user,
                {
                    method: "balance",
                    price: 1000
                }
            )

            bitrefillInvoice.transferId = transfer.id
            await bitrefillInvoice.save()

            return transfer
        }
        return null 
    }

    // TODO: pay bitrefillInvoice

    return payBitrefillInvoice(bitrefillInvoice)
    .then(response => {
        if (response.statusCode == 400) {
            console.log("Not enough funds: ", response.body)
            if (true) {
                // If balance is insufficient pay with lightning
                const data = {
                    products: [
                        {
                            "product_id": bitrefillInvoice.bitrefillProductId,
                            "quantity": 1,
                            "phone_number": bitrefillInvoice.order[0].phone_number,
                            "send_sms": true
                        }
                    ],
                    "payment_method": "lightning"
                }
            
                if (bitrefillInput.amount) {
                    data.products[0].value = bitrefillInput.amount
                } else if (packageId) {
                    data.products[0].package_id = bitrefillInput.packageId
                }
                return createBitrefillInvoice(data)
            }
            return response
        }  else {
            if (response.statusCode != 200) {
                console.log("Response Status: ", response.statusCode)
            }
            return response
        }
    }).then(async (response) => {
        if (response.statusCode == 200) {
            // TODO: Save bitrefillInvoice...
            if (user == null) {
                return null
            }
            const transfer = await persistTransfer(
                user,
                response.body.data.payment
            )
            bitrefillInvoice.transferId = transfer.id
            await bitrefillInvoice.save()

            return transfer
        } else {
            return null
        } 
    }).catch(error => {
        console.error("Failed to pay bitrefill invoice response: ", error)
        return null
    })
}

const persistTransfer = function (user, payment) {
    const amount = payment.price
    const serviceFee = Math.floor(amount*1/100)

    if (payment.method == "lightning") {
        return lightninNodeUtil.machankuraOutgoingLightningTranscation(
            user,
            amount,
            serviceFee,
            "Bitrefill",
            payment.address
        )
    } else if (payment.method == "balance") {
        // TODO: Create a Transfer object to Bitrefill...
        return db.Transfer.create({
            sourceId: user.id,
            amountInSatoshis: amount,
            feeInSatoshis: serviceFee,
            destinationParameter: "Bitrefill",
            status: "paid",
        })
    }
}